@@ -92,7 +92,7 @@ ncross.rq.fitXB <-
     }
     #-------------------------------------
     edf.rqXB <-function(obj, tau, id.coef, vMonot, vConc, pesiL1, output, 
-                        df.nc=TRUE, DFvalues=NULL, D.matrix, Bconstr, D1, D2, Y, X){
+                        df.nc=TRUE, DFvalues=NULL, D.matrix, Bconstr, D1, D2, Y, X, approx=FALSE){
       #versione per il calcolo edf in ..fitXB()
       #restituisce una lista dove la prima componente e' df.all, la seconda e' la somma per gruppi...
       #obj$DF.NEG e obj$DF.POS.. per i df accounting for noncross..
@@ -158,20 +158,31 @@ ncross.rq.fitXB <-
       H <- length(n.par) #n. smooths
       list.b <-tapply(b, id.coef, function(.x).x) 
       names(list.b)<- nomiGruppi
+      
+      if(approx){
+        df.all <- rep(1, length(b))
+        df.j<-tapply(df.all, id.coef, length)*.75
+        if(nomiGruppi[1]=="Xlin") df.j[1]<-pLin
+        if(is.matrix(df.j)) rownames(df.j)<-attr(id.coef, "nomi") else names(df.j)<-attr(id.coef, "nomi")
+        r<-list(edf.all=df.all, edf.j=df.j)
+        return(r)
+      }
+
       if(any(vConc!=0)) list.b1 <-list.b #creato list.b1 per l'eventuale concav (infatti list.b viene sovrascritto dalla monot)
       id.Xlin <- match("Xlin", nomiGruppi, 0)
       P<- if(nrow(D.matrix)==0) crossprod(D.matrix) else crossprod(drop(pesiL1)*D.matrix[(1:length(pesiL1)),,drop=FALSE])
+      
       #browser()
       
       if(any(vMonot!=0)){
-        wMon<- lapply(1:H, function(.x) matrix(0,n.par[.x]-1,n.par[.x]))
+        wMon<- vector("list", H) #lapply(1:H, function(.x) matrix(0,n.par[.x]-1,n.par[.x]))
         for(j in 1:H){ 
-          if(attr(Bconstr[[j]], "constr.fit")) {
-            list.b[[j + id.Xlin]] <-  drop(Bconstr[[j]] %*% list.b[[j + id.Xlin]])
-            wMon[[j]]  <- crossprod((10^6*(abs(diff(list.b[[j+ id.Xlin]], diff=1))<.00001))*D1[[j]])
-          } else {
-            wMon[[j]]  <- crossprod((10^6*(abs(drop(D1[[j]]%*%list.b[[j+ id.Xlin]]))<.00001))*D1[[j]])
-          }
+          if(vMonot[j]!=0){
+            if(attr(Bconstr[[j]], "constr.fit")) list.b[[j + id.Xlin]] <-  drop(Bconstr[[j]] %*% list.b[[j + id.Xlin]])
+              wMon[[j]]  <- crossprod((10^6*(abs(diff(list.b[[j+ id.Xlin]], diff=1))<.00001))*D1[[j]])
+            } else {
+              wMon[[j]]  <- matrix(0,n.par[j],n.par[j])
+            }
         }
         wMon<-if(length(wMon)==1) wMon[[1]] else do.call("blockdiag", wMon)
         if(nomiGruppi[1]=="Xlin") wMon <-blockdiag(matrix(0, pLin, pLin), wMon) 
@@ -179,10 +190,14 @@ ncross.rq.fitXB <-
       }
       
       if(any(vConc!=0)){
-        wConc<-lapply(1:H, function(.x) matrix(0,n.par[.x]-2,n.par[.x]))
-        for(j in 1:H){ 
-          if(attr(Bconstr[[j]], "constr.fit")) list.b1[[j+ id.Xlin]] <-  drop(Bconstr[[j]] %*% list.b1[[j+ id.Xlin]])
-          wConc[[j]] <- crossprod((10^6*(abs(diff(list.b1[[j+ id.Xlin]], diff=2))<.00001))*D2[[j]])
+        wConc<- vector("list", H) #lapply(1:H, function(.x) matrix(0,n.par[.x]-2,n.par[.x]))
+        for(j in 1:H){
+          if(vConc[j]!=0){
+            if(attr(Bconstr[[j]], "constr.fit")) list.b1[[j+ id.Xlin]] <-  drop(Bconstr[[j]] %*% list.b1[[j+ id.Xlin]])
+              wConc[[j]]  <- crossprod((10^6*(abs(diff(list.b1[[j+ id.Xlin]], diff=2))<.00001))*D2[[j]])
+            } else {
+            wConc[[j]]  <- matrix(0,n.par[j],n.par[j])
+            }
         }
         if(any(vConc!=0)) wConc <-if(length(wConc)==1) wConc[[1]] else do.call("blockdiag", wConc) 
         if(nomiGruppi[1]=="Xlin") wConc <-blockdiag(matrix(0, pLin, pLin), wConc) 
@@ -330,7 +345,11 @@ ncross.rq.fitXB <-
       }
     }
     #H <-length(B) #no. of smooth terms
-    B <-matrix(unlist(B), n, pSmooth) #matrix(unlist(B), nrow=3,byrow = FALSE)
+    #browser()
+    
+    #B <-matrix(unlist(B), n, pSmooth) #matrix(unlist(B), nrow=3,byrow = FALSE)
+    B <- do.call(cbind, B)
+    
     XB<-cbind(X,B)
     
     id.start.tau<-which.min(abs(taus-0.5))
@@ -399,6 +418,9 @@ ncross.rq.fitXB <-
     n.neg.taus<-length(neg.taus)
     XB.orig<-XB
     
+    
+    #browser()
+    
     if(any(lambda>0)){
       XB<- rbind(XB.orig, D.matrix.lambda)
     }
@@ -435,6 +457,7 @@ ncross.rq.fitXB <-
       #browser()
       
       for(i in 1:length(taus)){
+        #browser()
         o<-rq.fit.fnb(y=y, x=XB, tau=taus[i], eps=epss) 
         all.rho[i] <- sum(Rho(drop(o$residuals)[1:n], taus[i]))
         o$n<-n   #serve a edf.rqXB() 
@@ -445,7 +468,7 @@ ncross.rq.fitXB <-
         pesiL1<-sqrt(lambdasTutti/(pesiL1+.00001)) #(pesiL1[,k]+.00001)
         df.start.tau <- edf.rqXB(o, tau=taus[i], id.coef=id.df, vMonot = monotone, vConc=concave, pesiL1=pesiL1,
                                output=list(...)$df.option, df.nc=FALSE, DFvalues=NULL, D.matrix, Bconstr, D1, D2, 
-                               Y=y[1:n], X=XB.orig)
+                               Y=y[1:n], X=XB.orig, approx=TRUE)
         
         all.edf[,i]<- df.start.tau$edf.all
         group.edf[,i]<-df.start.tau$edf.j
@@ -495,6 +518,7 @@ ncross.rq.fitXB <-
     lambdasTutti<- rep(lambdaM[,paste(start.tau)], sapply(D.list, nrow)) #controlla se serve -1*dropcList
     if(!is.null(pLin) && pLin>0) lambdasTutti<-c(rep(0, pLin),lambdasTutti)
     pesiL1<-sqrt(lambdasTutti/(pesiL1+.00001)) #(pesiL1[,k]+.00001)
+
     df.start.tau <- edf.rqXB(o.start, tau=start.tau, id.coef=id.df, vMonot = monotone, vConc=concave, pesiL1=pesiL1,
                              output=list(...)$df.option, df.nc=FALSE, DFvalues=NULL, D.matrix, 
                              Bconstr, D1, D2, Y=y[1:n], X=XB.orig)
@@ -544,9 +568,6 @@ ncross.rq.fitXB <-
           RR<- Ident
           rr<- b.start + eps
         }
-        
-        #browser()
-        
         #DF per NONcrossing
         DF.POS <- matrix(,nrow(RR),n.pos.taus)
         colnames(DF.POS) <- paste(pos.taus)
@@ -604,11 +625,17 @@ ncross.rq.fitXB <-
             if(any(lambda>0)) XB<-rbind(XB.orig, D.matrix.lambda)
             if(lambda.ridge>0) XB<-rbind(XB, lambda.ridge*diag(ncol(XB))) #a small ridge penalty
           }
+          
+          #browser()
+          
           o<-rq.fit(x=XB, y=y, tau=pos.taus[i], method="fnc", R=RR, r=rr)
           #o<-rq.fit.fnc(x=XB, y=y, tau=pos.taus[i], method="fnc", R=RR, r=rr, eps=myeps)
           if(!all(is.finite(o$coefficients))) warning(paste("Some NA estimate in the quantile tau =", pos.taus[i], sep="" ))
           
           o$rho<-sum(Rho(o$residuals[1:n], pos.taus[i]))
+          
+          #browser()
+          
           #estrai fitted e residuals
           FIT.POS[,i]<-o$fitted.values[1:n]
           RES.POS[,i]<-o$residuals[1:n]
@@ -748,13 +775,16 @@ ncross.rq.fitXB <-
             if(any(lambda>0)) XB<-rbind(XB.orig, D.matrix.lambda)
             if(lambda.ridge>0) XB<-rbind(XB, lambda.ridge*diag(ncol(XB))) #a small ridge penalty
           }
-          o<-rq.fit(x=XB,y=y,tau=neg.taus[i],method="fnc",R=RR,r=rr)
+          
+          #browser()
+          
+          o<-rq.fit.fnc(x=XB,y=y,tau=neg.taus[i],R=RR,r=rr)
           #o<-rq.fit.fnc(x=XB,y=y,tau=neg.taus[i], R=RR, r=rr, eps=myeps)
           ##########==============================================================
           
           if(!all(is.finite(o$coefficients)))  warning(paste("At least one estimate is NA in the quantile curve tau = ", neg.taus[i], sep="" ))
           o$rho<-sum(Rho(o$residuals[1:n], neg.taus[i]))
-          FIT.NEG[,i]<-o$fitted.values[1:n]
+          FIT.NEG[,i]<- y[1:n]- o$residuals[1:n] #o$fitted.values[1:n]
           RES.NEG[,i]<-o$residuals[1:n]
           #df.neg.tau[i] <- sum(abs(o$residuals[1:n])<=.000001)
           rho.neg.tau[i] <- o$rho #sum(Rho(o$residuals[1:n], neg.taus[i]))
